# !/usr/bin/python3
# -*- coding: UTF-8 -*-

import re

import pytest
from playwright.sync_api import Page, expect


@pytest.fixture(scope="function", autouse=True)
def before_each_after_each(page: Page):
    print("\nbeforeEach")
    # Go to the starting url before each test.
    page.goto("https://playwright.dev/")
    yield
    print("\nafterEach")


# Pages are isolated between tests due to the Browser Context, which is equivalent to a brand new browser profile,
# where every test gets a fresh environment, even when multiple tests run in a single Browser.
def test_homepage_has_playwright_in_title_and_get_started_link_linking_to_the_intro_page(page: Page):
    # page.goto("https://playwright.dev/")

    # Expect a title "to contain" a substring.
    expect(page).to_have_title(re.compile("Playwright"))  # Assertions

    # create a locator
    get_started = page.get_by_role("link", name="Get started")  # Locators

    # Expect an attribute "to be strictly equal" to the value.
    expect(get_started).to_have_attribute("href", "/docs/intro")

    # Click the get started link.
    get_started.click()

    # Expects the URL to contain intro.
    expect(page).to_have_url(re.compile(".*intro"))
